/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.geoviewer.jxmap.map;

import com.vividsolutions.jts.geom.Geometry;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Point2D;
import nl.cloudfarming.client.geoviewer.Geometrical;
import nl.cloudfarming.client.geoviewer.Palette;
import nl.cloudfarming.client.geoviewer.jxmap.render.GeometryRenderer;
import nl.cloudfarming.client.geoviewer.jxmap.render.GeometryRendererFactory;
import nl.cloudfarming.client.util.GeometryUtil;
import org.jdesktop.swingx.JXMapViewer;
import org.jdesktop.swingx.mapviewer.GeoPosition;
import org.netbeans.api.visual.widget.Widget;

/**
 * An ItemWidget is a object on the map which is used for interaction from the
 * user with a dynamic object on the map (a {@link LayerItem}). Someone can see
 * a toolTip for a dynamic object and can use a context menu (right mouse click
 * on an object).
 *
 * @author Maarten Hollenga
 */
public class GeometricalWidget extends Widget {

    private static final int ICON_SIZE = 20;
    private String id;
    private final Geometrical geometrical;
    private final Palette palette;
    private static final double DISTANCE = 0.00001;

    public void setId(String id) {
        this.id = id;
    }

    public String getId() {
        return id;
    }

    /**
     * Initialize the widget for a dynamic object on the specified location with
     * a toolTip and a context menu.
     *
     * @param scene A class that keeps associations between widgets and the
     * objects that model them
     * @param geometrical geometrical object to display on the map
     * @param palette palette with colors
     */
    public GeometricalWidget(final MapScene scene, final Geometrical geometrical, Palette palette) {
        super(scene);
        if (geometrical == null || geometrical.getGeometry() == null) {
            throw new IllegalArgumentException("argument geometrical or its geometry cannot be null");
        }
        this.palette = palette;
        this.geometrical = geometrical;
        this.id = new Long(geometrical.getId()).toString();
        setPreferredSize(new Dimension(ICON_SIZE, ICON_SIZE));
        setToolTipText(geometrical.getTooltipText());
        setForeground(this.palette.getColorForState(getState()));
    }

    private JXMapViewer getMapViewer() {
        return ((MapScene) getScene()).getMapViewer();
    }

    @Override
    protected void paintWidget() {
        //
        // set the preferred widget bounds based on the geometry and current state of the mapviewer
        //
        Rectangle clientArea = GeometryRenderer.getViewport(geometrical.getGeometry(), getMapViewer());
        clientArea = (clientArea != null) ? clientArea : super.calculateClientArea();
        setPreferredBounds(clientArea);
        //
        // prepare the graphics and clip to the widget bounds to minimize paint effort
        //
        Graphics2D g = (Graphics2D) getGraphics().create();
        g.clip(getPreferredBounds());
        g.setColor(palette.getColorForState(getState()));
        //
        // dispatch painting of the geometry to the geometry renderer
        //
        GeometryRenderer renderer = GeometryRendererFactory.getRenderer(geometrical.getGeometry());
        renderer.render(geometrical.getGeometry(), getMapViewer(), g);
        //
        // dispose of the graphics copy
        //
        g.dispose();
    }

    /**
     * Determines if a widget isHit. A widget is hit when a point lays between
     * the bounds of the widget and: - The localLocation intersects the widgets
     * GeoPostition - the localLocation is within a distance of DISTANCE of
     * GeoPosition
     *
     * @param localLocation the Point which possibly hits the widget
     * @return if the widget is hit
     */
    @Override
    public boolean isHitAt(Point localLocation) {
        if (isVisible() && getBounds().contains(localLocation)) {
            Geometry geometry = geometrical.getGeometry();
            GeoPosition geoPosition = convertToGeoPosition(localLocation, getMapViewer());
            com.vividsolutions.jts.geom.Point pnt = GeometryUtil.getPoint(geoPosition.getLongitude(), geoPosition.getLatitude());
            return geometry.intersects(pnt) || geometry.isWithinDistance(pnt, DISTANCE);

        } else {
            return false;
        }
    }

    /**
     * TODO: put in util (WILL THROW A RUNTIME EXCEPTION FOR SOME REASON!)
     *
     * Convert pixel coordinate to WGS84 coordinate
     *
     * @param localLocation
     * @param mapViewer
     * @return WGS84 GeoPosition
     */
    public static GeoPosition convertToGeoPosition(java.awt.Point localLocation, JXMapViewer mapViewer) {
        Rectangle viewportBounds = mapViewer.getViewportBounds();
        int mapX = (int) (viewportBounds.getX() + localLocation.getX());
        int mapY = (int) (viewportBounds.getY() + localLocation.getY());
        Point2D newPoint = new java.awt.Point(mapX, mapY);
        GeoPosition geoPosition = mapViewer.getTileFactory().pixelToGeo(newPoint, mapViewer.getZoom());
        return geoPosition;
    }
}
